#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-
# generated by wxGlade 0.6 on Mon Dec 03 03:12:55 2007


#Guide to adding new fits:
#1.Add panel to wxg and regenerate GUI
#2.add elif clause to fitData
#3. update pagedict dictionary
#4.update documentation

import wx
import numpy,matplotlib
from matplotlib.widgets import SpanSelector
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg,FigureManager

# begin wxGlade: extracode
# end wxGlade

class NavBar(NavigationToolbar2WxAgg):
    def __init__(self,*arga,**kwargs):
        NavigationToolbar2WxAgg.__init__(self,*arga,**kwargs)

        self.DeleteToolByPos(self.GetToolsCount()-2)

        self.Bind(wx.EVT_TOOL,self.onTool,id=self._NTB2_ZOOM)
        self.Bind(wx.EVT_TOOL,self.onTool,id=self._NTB2_PAN)

        self.__toolfuncs=[]

    def addToolListener(self,func):
        """
        adds a function to listen for one of the togglable tools to be turned on or off
        func should have the signature func(state), where state will be True if the
        tool is activated, or False if it is deactivated
        """
        self.__toolfuncs.append(func)

    def removeToolListener(self,func):
        self.__toolfuncs.remove(func)

    def clearToolListeners(self):
        self.__toolfuncs=[]

    def getToolState(self):
        return self.GetToolState(self._NTB2_ZOOM) or self.GetToolState(self._NTB2_PAN)

    def onTool(self,event):
        #state is true if the Tool is turning on
        state=self.GetToolState(event.GetId())
        for f in self.__toolfuncs:
            f(state)
        event.Skip()
        
    def clickZoom(self):
        state=self.GetToolState(self._NTB2_ZOOM)
        self.zoom()
        self.ToggleTool(self._NTB2_ZOOM, not state)
        for f in self.__toolfuncs:
            f(not state)
            
    def clickPan(self):
        state=self.GetToolState(self._NTB2_PAN)
        self.pan()
        self.ToggleTool(self._NTB2_PAN, not state)
        for f in self.__toolfuncs:
            f(not state)
            
class WheelZoomer(object):
    def __init__(self,canvas,zoomfactor=1.5):
        self.fc=canvas
        self.zoomfactor=zoomfactor
        canvas.Bind(wx.EVT_MOUSEWHEEL,self.onWheel)
        
    def onWheel(self, event):
        axs=self.fc.figure.axes
        if axs:
            ax=axs[-1]
            xdata,ydata = ax.transData.inverse_xy_tup((event.GetX(),ax.figure.bbox.height() - event.GetY()))
            
            xl,xu=ax.get_xlim()
            dx=xu-xl
            yl,yu=ax.get_ylim()
            dy=yu-yl
            if event.m_wheelRotation > 0:
                dx/=self.zoomfactor
                dy/=self.zoomfactor
            else:
                dx*=self.zoomfactor
                dy*=self.zoomfactor
                
            ax.set_xlim(xdata-dx/2,xdata+dx/2)
            ax.set_ylim(ydata-dy/2,ydata+dy/2)
            self.fc.draw()
        
class FitPanel(wx.Panel):
    """
    wx Panel used for function fitting - has the settable variables:
    self.selalpha
    self.selfacecolor
    self.seledgecolor
    self.datastyle
    self.resstyle
    self.fitstyle
    self.log
    """
    def __init__(self,x,y,usepages,*args, **kwds):
        """
        x and y are the data to be fit. x can be none, which will result in a
        linear [0,len(y)) axis.
        any further arguments will be processed by wx.Panel.__init__
        usepages should be a sequence if strings or None - if sequence, it specifies
        which fitting techniques to use - options are:
        'spline' : Fitted Smoothed Spline
        'poly' : Simple,Chebychev, and Legendre Polynomials
        'power':  Power Law
        'bb': Planck Blackbody radiation law
        'gauss': Gaussian / Normal distribution function
        'lorentz': Lorentzian function
        'voigt' : voigt (Lorenzan conv Gaussian) function
        '2gauss':double Gaussian function
        '2-power':two-power density models
        """
        #process non wx.Panel arguments
        if x is None:
            self.__x=numpy.arange(len(y))
        else:
            self.__x=numpy.array(x)
        self.__y=numpy.array(y)
        self.__selections = None
        self.__sstool = None
        self.__drawcode=0
        
        self.selalpha=0.3
        self.selfacecolor='r'
        self.seledgecolor='k'
        self.datastyle='b'
        self.resstyle='r'
        self.fitstyle='g'
        self.log=False
        
        #initial figure setup and data draw
        self.fig = Figure()
        self.axes = self.fig.add_subplot(111)
        
        #wait on drawing until after canvas is made
        self._updatePreview(drawtocanvas=False)
        
        # begin wxGlade: FitPanel.__init__
        kwds["style"] = wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.fitNotebook = wx.Notebook(self, -1, style=0)
        self.twoPowerFitPane = wx.Panel(self.fitNotebook, -1)
        self.doubleGaussFitPane = wx.Panel(self.fitNotebook, -1)
        self.voigtFitPane = wx.Panel(self.fitNotebook, -1)
        self.lorentzFitPane = wx.Panel(self.fitNotebook, -1)
        self.gaussFitPane = wx.Panel(self.fitNotebook, -1)
        self.blackbodyFitPane = wx.Panel(self.fitNotebook, -1)
        self.powerLawFitPane = wx.Panel(self.fitNotebook, -1)
        self.polyFitPane = wx.Panel(self.fitNotebook, -1)
        self.splineFitPane = wx.Panel(self.fitNotebook, -1)
        self.sizer_6_staticbox = wx.StaticBox(self.blackbodyFitPane, -1, "Tolerances")
        self.sizer_1_staticbox = wx.StaticBox(self, -1, "Preview")
        self.selectionRadioBox = wx.RadioBox(self, -1, "Selection", choices=["All", "Include", "Exclude"], majorDimension=0, style=wx.RA_SPECIFY_ROWS)
        self.prevFitButton = wx.Button(self, -1, "Show Fit")
        self.prevResidualsButton = wx.Button(self, -1, "Show Residuals")
        self.prevDFRButton = wx.Button(self, -1, "Show All")
        self.prevResetButton = wx.Button(self, -1, "Reset")
        self.previewCanvas = FigureCanvas(self, -1, self.fig)
        self.previewNavBar = NavBar(self.previewCanvas)
        self.fitLabel = wx.StaticText(self, -1, "Choose a fitting function")
        self.label_6 = wx.StaticText(self.splineFitPane, -1, "Degree:")
        self.splineDegTextCtrl = wx.TextCtrl(self.splineFitPane, -1, "3", style=wx.TE_PROCESS_ENTER)
        self.label_2 = wx.StaticText(self.splineFitPane, -1, "Smoothing:")
        self.splineSmoothingTextCtrl = wx.TextCtrl(self.splineFitPane, -1, "1.0", style=wx.TE_PROCESS_ENTER)
        self.label_8 = wx.StaticText(self.polyFitPane, -1, "Degree:")
        self.polyDegTextCtrl = wx.TextCtrl(self.polyFitPane, -1, "3", style=wx.TE_PROCESS_ENTER)
        self.polyTypeRadioBox = wx.RadioBox(self.polyFitPane, -1, "Type", choices=["Simple", "Legendre", "Chebyshev T", "U", "S", "C"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        self.label_1_copy = wx.StaticText(self.polyFitPane, -1, "Tolerance:")
        self.polyTolTextCtrl = wx.TextCtrl(self.polyFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_16 = wx.StaticText(self.powerLawFitPane, -1, "A*x^alpha")
        self.powerSlopeLabel = wx.StaticText(self.powerLawFitPane, -1, "Alpha:")
        self.powerSlopeTextCtrl = wx.TextCtrl(self.powerLawFitPane, -1, "1", style=wx.TE_PROCESS_ENTER)
        self.label_9 = wx.StaticText(self.powerLawFitPane, -1, "A:")
        self.powerOffTextCtrl = wx.TextCtrl(self.powerLawFitPane, -1, "1", style=wx.TE_PROCESS_ENTER)
        self.label_1_copy_2 = wx.StaticText(self.powerLawFitPane, -1, "Tolerance:")
        self.powerTolTextCtrl = wx.TextCtrl(self.powerLawFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.bbTypeRadioBox = wx.RadioBox(self.blackbodyFitPane, -1, "x-axis variable", choices=["Wavelength", "Frequency"], majorDimension=0, style=wx.RA_SPECIFY_ROWS)
        self.label_23 = wx.StaticText(self.blackbodyFitPane, -1, "A")
        self.bbInitA0 = wx.TextCtrl(self.blackbodyFitPane, -1, "")
        self.label_10 = wx.StaticText(self.blackbodyFitPane, -1, "Initial Temperature:")
        self.bbInitTempTextCtrl = wx.TextCtrl(self.blackbodyFitPane, -1, "5000", style=wx.TE_PROCESS_ENTER)
        self.label_3 = wx.StaticText(self.blackbodyFitPane, -1, "ftol")
        self.bbFtolTextCtrl = wx.TextCtrl(self.blackbodyFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_4 = wx.StaticText(self.blackbodyFitPane, -1, "xtol")
        self.bbXtolTextCtrl = wx.TextCtrl(self.blackbodyFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_7 = wx.StaticText(self.gaussFitPane, -1, "A exp(-(x+mu)^2/2 sigma)/sqrt(2 pi sigma^2)+C")
        self.label_5 = wx.StaticText(self.gaussFitPane, -1, "A (normalization)")
        self.label_11 = wx.StaticText(self.gaussFitPane, -1, "sigma (Standard Dev)")
        self.label_12 = wx.StaticText(self.gaussFitPane, -1, "mu (offset)")
        self.gaussATextCtrl = wx.TextCtrl(self.gaussFitPane, -1, "1")
        self.gaussSigTextCtrl = wx.TextCtrl(self.gaussFitPane, -1, "1")
        self.gaussMuCheckBox = wx.CheckBox(self.gaussFitPane, -1, "Float mu?")
        self.gaussMuTextCtrl = wx.TextCtrl(self.gaussFitPane, -1, "1")
        self.label_13_copy_copy_copy = wx.StaticText(self.gaussFitPane, -1, "Vertical Offset:")
        self.gaussCCheckBox = wx.CheckBox(self.gaussFitPane, -1, "Float C?")
        self.gaussCTextCtrl = wx.TextCtrl(self.gaussFitPane, -1, "0")
        self.label_1 = wx.StaticText(self.gaussFitPane, -1, "Tolerance")
        self.gaussTolTextCtrl = wx.TextCtrl(self.gaussFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_7_copy_1 = wx.StaticText(self.lorentzFitPane, -1, "A xs/pi (x^2+xs^2)+C")
        self.label_5_copy_1 = wx.StaticText(self.lorentzFitPane, -1, "A (normalization)")
        self.label_11_copy_1 = wx.StaticText(self.lorentzFitPane, -1, "xs (Lorentzian scale factor)")
        self.label_12_copy_1 = wx.StaticText(self.lorentzFitPane, -1, "mu (offset)")
        self.lorATextCtrl = wx.TextCtrl(self.lorentzFitPane, -1, "1")
        self.lorScaleTextCtrl = wx.TextCtrl(self.lorentzFitPane, -1, "1")
        self.lorMuCheckBox = wx.CheckBox(self.lorentzFitPane, -1, "Float mu?")
        self.lorMuTextCtrl = wx.TextCtrl(self.lorentzFitPane, -1, "1")
        self.label_13_copy_copy = wx.StaticText(self.lorentzFitPane, -1, "Vertical Offset:")
        self.lorCCheckBox = wx.CheckBox(self.lorentzFitPane, -1, "Float C?")
        self.lorCTextCtrl = wx.TextCtrl(self.lorentzFitPane, -1, "0")
        self.label_1_copy_3 = wx.StaticText(self.lorentzFitPane, -1, "Tolerance")
        self.lorTolTextCtrl = wx.TextCtrl(self.lorentzFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_7_copy_2 = wx.StaticText(self.voigtFitPane, -1, "Convolved Gaussian and Lorentzian")
        self.label_5_copy_1_copy = wx.StaticText(self.voigtFitPane, -1, "A (overall normalization)")
        self.label_11_copy_2 = wx.StaticText(self.voigtFitPane, -1, "sigma (Standard Dev)")
        self.label_11_copy_1_copy = wx.StaticText(self.voigtFitPane, -1, "xs (Lorentzian scale factor)")
        self.label_12_copy_1_copy = wx.StaticText(self.voigtFitPane, -1, "mu (offset)")
        self.voigtATextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.voigtSigTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.voigtScaleTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.voigtMuCheckBox = wx.CheckBox(self.voigtFitPane, -1, "Float mu?")
        self.voigtMuTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.label_14 = wx.StaticText(self.voigtFitPane, -1, "G (gaussian\n normalization)")
        self.label_15 = wx.StaticText(self.voigtFitPane, -1, "L (lorentzian\n normalization)")
        self.label_13_copy = wx.StaticText(self.voigtFitPane, -1, "Vertical Offset:")
        self.voigtCCheckBox = wx.CheckBox(self.voigtFitPane, -1, "Float C?")
        self.voigtCTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "0")
        self.voigtGTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.voigtLTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1")
        self.voigtFixACheckBox = wx.CheckBox(self.voigtFitPane, -1, "fix A?")
        self.voigtFixGCheckBox = wx.CheckBox(self.voigtFitPane, -1, "fix G?")
        self.voigtFixLCheckBox = wx.CheckBox(self.voigtFitPane, -1, "fix L?")
        self.label_1_copy_4 = wx.StaticText(self.voigtFitPane, -1, "Tolerance")
        self.voigtTolTextCtrl = wx.TextCtrl(self.voigtFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_7_copy = wx.StaticText(self.doubleGaussFitPane, -1, "abs(A) exp(-(x+muA)^2/2 sigmaA^2)/sqrt(2 pi sigmaA)-abs(B) exp(-(x+muB)^2/2 sigmaB^2)/sqrt(2 pi sigmaB)+C")
        self.label_5_copy = wx.StaticText(self.doubleGaussFitPane, -1, "A (normalization)")
        self.label_11_copy = wx.StaticText(self.doubleGaussFitPane, -1, "sigma (Standard Dev)")
        self.label_12_copy = wx.StaticText(self.doubleGaussFitPane, -1, "mu (offset)")
        self.dgaussATextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1")
        self.dgaussSigATextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1")
        self.dgaussMuACheckBox = wx.CheckBox(self.doubleGaussFitPane, -1, "Float muA?")
        self.dgaussMuATextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1")
        self.singleFitAButton = wx.Button(self.doubleGaussFitPane, -1, "Single Gaussian to A")
        self.dgaussBTextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "-1")
        self.dgaussSigBTextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1")
        self.dgaussMuBCheckBox = wx.CheckBox(self.doubleGaussFitPane, -1, "Float muB?")
        self.dgaussMuBTextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1")
        self.singleFitBButton = wx.Button(self.doubleGaussFitPane, -1, "Single Gaussian to B")
        self.label_13 = wx.StaticText(self.doubleGaussFitPane, -1, "Vertical Offset:")
        self.dgaussCCheckBox = wx.CheckBox(self.doubleGaussFitPane, -1, "Float C?")
        self.dgaussCTextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "0")
        self.label_1_copy_1 = wx.StaticText(self.doubleGaussFitPane, -1, "Tolerance")
        self.dgaussTolTextCtrl = wx.TextCtrl(self.doubleGaussFitPane, -1, "1E-8", style=wx.TE_PROCESS_ENTER)
        self.label_17 = wx.StaticText(self.twoPowerFitPane, -1, "2-power model: A/[x^alpha (x+xc)^(beta-alpha) ]")
        self.label_18 = wx.StaticText(self.twoPowerFitPane, -1, "A")
        self.twoPowerATextCtrl = wx.TextCtrl(self.twoPowerFitPane, -1, "1")
        self.twoPowerFixedACheckBox = wx.CheckBox(self.twoPowerFitPane, -1, "Fixed?")
        self.label_19 = wx.StaticText(self.twoPowerFitPane, -1, "xc")
        self.twoPowerXcTextCtrl = wx.TextCtrl(self.twoPowerFitPane, -1, "1")
        self.twoPowerFixedXcCheckBox = wx.CheckBox(self.twoPowerFitPane, -1, "Fixed?")
        self.label_20 = wx.StaticText(self.twoPowerFitPane, -1, "alpha")
        self.twoPowerAlphaTextCtrl = wx.TextCtrl(self.twoPowerFitPane, -1, "1")
        self.twoPowerFixedAlphaCheckBox = wx.CheckBox(self.twoPowerFitPane, -1, "Fixed?")
        self.label_21 = wx.StaticText(self.twoPowerFitPane, -1, "beta")
        self.twoPowerBetaTextCtrl = wx.TextCtrl(self.twoPowerFitPane, -1, "3")
        self.twoPowerFixedBetaCheckBox = wx.CheckBox(self.twoPowerFitPane, -1, "Fixed?")
        self.label_22 = wx.StaticText(self.twoPowerFitPane, -1, "Tolerance?")
        self.twoPowerTolTextCtrl = wx.TextCtrl(self.twoPowerFitPane, -1, "1e-8")
        self.fitOutputText = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE|wx.TE_READONLY)

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_RADIOBOX, self.onSelectChange, self.selectionRadioBox)
        self.Bind(wx.EVT_BUTTON, self.onPrevFit, self.prevFitButton)
        self.Bind(wx.EVT_BUTTON, self.onPrevRes, self.prevResidualsButton)
        self.Bind(wx.EVT_BUTTON, self.onDFR, self.prevDFRButton)
        self.Bind(wx.EVT_BUTTON, self.onPrevReset, self.prevResetButton)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.splineDegTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.splineSmoothingTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.polyDegTextCtrl)
        self.Bind(wx.EVT_RADIOBOX, self.onParamEnter, self.polyTypeRadioBox)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.polyTolTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.powerSlopeTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.powerOffTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.powerTolTextCtrl)
        self.Bind(wx.EVT_RADIOBOX, self.onParamEnter, self.bbTypeRadioBox)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.bbInitTempTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.bbFtolTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.bbXtolTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.gaussTolTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.lorTolTextCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.voigtTolTextCtrl)
        self.Bind(wx.EVT_BUTTON, self.onSingleFitA, self.singleFitAButton)
        self.Bind(wx.EVT_BUTTON, self.onSingleFitB, self.singleFitBButton)
        self.Bind(wx.EVT_TEXT_ENTER, self.onParamEnter, self.dgaussTolTextCtrl)
        # end wxGlade
        
        #register key events
        self.previewCanvas.Bind(wx.EVT_CHAR, self.onChar)
        
        #delete pages that aren't requested
        if usepages is not None:
            pagedict={'spline':0,
            'poly':1,
            'power':2,
            'bb':3,
            'gauss':4,
            'lorentz':5,
            'voigt' :6,
            '2gauss':7,
            '2-power':8}
            delpages=range(9)
            try:
                for p in usepages:
                    delpages.remove(pagedict[p])
            except KeyError:
                raise ValueError('unrecognized fit type request: '+p)
            for i in sorted(delpages,reverse=True):
                self.fitNotebook.DeletePage(i)
                
        self.previewNavBar.addToolListener(self.onToolToggle)
        WheelZoomer(self.previewCanvas)
        
        self.__smartInitialParams(self.__x,self.__y)
        
        self.previewCanvas.draw()
        
    def __set_properties(self):
        # begin wxGlade: FitPanel.__set_properties
        self.selectionRadioBox.SetSelection(0)
        self.previewCanvas.SetMinSize((700,300))
        self.polyTypeRadioBox.SetSelection(0)
        self.bbTypeRadioBox.SetSelection(0)
        self.voigtFixACheckBox.SetValue(1)
        self.twoPowerFixedAlphaCheckBox.SetValue(1)
        self.twoPowerFixedBetaCheckBox.SetValue(1)
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: FitPanel.__do_layout
        sizer_7 = wx.BoxSizer(wx.VERTICAL)
        sizer_11 = wx.BoxSizer(wx.VERTICAL)
        sizer_12 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_6 = wx.GridSizer(4, 3, 0, 0)
        sizer_9_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_10_copy = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_4_copy = wx.FlexGridSizer(4, 5, 0, 0)
        sizer_9_copy_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_10_copy_2 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_5 = wx.FlexGridSizer(3, 5, 0, 0)
        sizer_9_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_10_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_4_copy_1 = wx.FlexGridSizer(3, 4, 0, 0)
        sizer_9 = wx.BoxSizer(wx.VERTICAL)
        sizer_10 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_4 = wx.FlexGridSizer(3, 4, 0, 0)
        sizer_16 = wx.BoxSizer(wx.VERTICAL)
        sizer_6 = wx.StaticBoxSizer(self.sizer_6_staticbox, wx.HORIZONTAL)
        sizer_17 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_3 = wx.FlexGridSizer(5, 2, 0, 0)
        grid_sizer_2 = wx.FlexGridSizer(3, 2, 0, 0)
        grid_sizer_1 = wx.FlexGridSizer(2, 2, 0, 0)
        sizer_1 = wx.StaticBoxSizer(self.sizer_1_staticbox, wx.HORIZONTAL)
        sizer_8 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_5 = wx.BoxSizer(wx.VERTICAL)
        sizer_2.Add(self.selectionRadioBox, 0, 0, 0)
        sizer_5.Add(self.prevFitButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_5.Add(self.prevResidualsButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_5.Add(self.prevDFRButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_5.Add(self.prevResetButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_2.Add(sizer_5, 0, wx.EXPAND, 0)
        sizer_1.Add(sizer_2, 0, wx.EXPAND, 0)
        sizer_8.Add(self.previewCanvas, 1, wx.EXPAND, 0)
        sizer_8.Add(self.previewNavBar, 0, wx.EXPAND, 0)
        sizer_1.Add(sizer_8, 1, wx.EXPAND, 0)
        sizer_7.Add(sizer_1, 5, wx.EXPAND, 0)
        sizer_7.Add(self.fitLabel, 0, 0, 0)
        grid_sizer_1.Add(self.label_6, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_1.Add(self.splineDegTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_1.Add(self.label_2, 0, 0, 0)
        grid_sizer_1.Add(self.splineSmoothingTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        self.splineFitPane.SetSizer(grid_sizer_1)
        grid_sizer_2.Add(self.label_8, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_2.Add(self.polyDegTextCtrl, 0, 0, 0)
        grid_sizer_2.Add((20, 5), 0, 0, 0)
        grid_sizer_2.Add(self.polyTypeRadioBox, 0, 0, 0)
        grid_sizer_2.Add(self.label_1_copy, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_2.Add(self.polyTolTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        self.polyFitPane.SetSizer(grid_sizer_2)
        grid_sizer_3.Add((1, 1), 0, 0, 0)
        grid_sizer_3.Add(self.label_16, 0, 0, 0)
        grid_sizer_3.Add(self.powerSlopeLabel, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_3.Add(self.powerSlopeTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_3.Add(self.label_9, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_3.Add(self.powerOffTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_3.Add(self.label_1_copy_2, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_3.Add(self.powerTolTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        self.powerLawFitPane.SetSizer(grid_sizer_3)
        sizer_16.Add(self.bbTypeRadioBox, 0, 0, 0)
        sizer_17.Add(self.label_23, 0, 0, 0)
        sizer_17.Add(self.bbInitA0, 0, 0, 0)
        sizer_17.Add(self.label_10, 0, 0, 0)
        sizer_17.Add(self.bbInitTempTextCtrl, 0, 0, 0)
        sizer_16.Add(sizer_17, 1, wx.EXPAND, 0)
        sizer_6.Add(self.label_3, 0, wx.LEFT|wx.RIGHT|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_6.Add(self.bbFtolTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(self.label_4, 0, wx.LEFT|wx.RIGHT|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_6.Add(self.bbXtolTextCtrl, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_16.Add(sizer_6, 1, wx.EXPAND, 0)
        self.blackbodyFitPane.SetSizer(sizer_16)
        sizer_9.Add(self.label_7, 0, 0, 0)
        grid_sizer_4.Add(self.label_5, 0, 0, 0)
        grid_sizer_4.Add(self.label_11, 0, 0, 0)
        grid_sizer_4.Add((20, 20), 0, 0, 0)
        grid_sizer_4.Add(self.label_12, 0, 0, 0)
        grid_sizer_4.Add(self.gaussATextCtrl, 0, 0, 0)
        grid_sizer_4.Add(self.gaussSigTextCtrl, 0, 0, 0)
        grid_sizer_4.Add(self.gaussMuCheckBox, 0, wx.ALIGN_RIGHT, 0)
        grid_sizer_4.Add(self.gaussMuTextCtrl, 0, 0, 0)
        grid_sizer_4.Add((20, 20), 0, 0, 0)
        grid_sizer_4.Add(self.label_13_copy_copy_copy, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4.Add(self.gaussCCheckBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4.Add(self.gaussCTextCtrl, 0, 0, 0)
        sizer_9.Add(grid_sizer_4, 1, wx.EXPAND, 0)
        sizer_10.Add(self.label_1, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10.Add(self.gaussTolTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_9.Add(sizer_10, 0, wx.EXPAND, 0)
        self.gaussFitPane.SetSizer(sizer_9)
        sizer_9_copy_1.Add(self.label_7_copy_1, 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.label_5_copy_1, 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.label_11_copy_1, 0, 0, 0)
        grid_sizer_4_copy_1.Add((20, 20), 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.label_12_copy_1, 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.lorATextCtrl, 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.lorScaleTextCtrl, 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.lorMuCheckBox, 0, wx.ALIGN_RIGHT, 0)
        grid_sizer_4_copy_1.Add(self.lorMuTextCtrl, 0, 0, 0)
        grid_sizer_4_copy_1.Add((20, 20), 0, 0, 0)
        grid_sizer_4_copy_1.Add(self.label_13_copy_copy, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4_copy_1.Add(self.lorCCheckBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4_copy_1.Add(self.lorCTextCtrl, 0, 0, 0)
        sizer_9_copy_1.Add(grid_sizer_4_copy_1, 1, wx.EXPAND, 0)
        sizer_10_copy_1.Add(self.label_1_copy_3, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy_1.Add(self.lorTolTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_9_copy_1.Add(sizer_10_copy_1, 0, wx.EXPAND, 0)
        self.lorentzFitPane.SetSizer(sizer_9_copy_1)
        sizer_9_copy_2.Add(self.label_7_copy_2, 0, 0, 0)
        grid_sizer_5.Add(self.label_5_copy_1_copy, 0, 0, 0)
        grid_sizer_5.Add(self.label_11_copy_2, 0, 0, 0)
        grid_sizer_5.Add(self.label_11_copy_1_copy, 0, 0, 0)
        grid_sizer_5.Add((20, 20), 0, 0, 0)
        grid_sizer_5.Add(self.label_12_copy_1_copy, 0, 0, 0)
        grid_sizer_5.Add(self.voigtATextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtSigTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtScaleTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtMuCheckBox, 0, wx.ALIGN_RIGHT, 0)
        grid_sizer_5.Add(self.voigtMuTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.label_14, 0, 0, 0)
        grid_sizer_5.Add(self.label_15, 0, 0, 0)
        grid_sizer_5.Add(self.label_13_copy, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_5.Add(self.voigtCCheckBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_5.Add(self.voigtCTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtGTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtLTextCtrl, 0, 0, 0)
        grid_sizer_5.Add(self.voigtFixACheckBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_5.Add(self.voigtFixGCheckBox, 0, 0, 0)
        grid_sizer_5.Add(self.voigtFixLCheckBox, 0, 0, 0)
        sizer_9_copy_2.Add(grid_sizer_5, 1, wx.EXPAND, 0)
        sizer_10_copy_2.Add(self.label_1_copy_4, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy_2.Add(self.voigtTolTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_9_copy_2.Add(sizer_10_copy_2, 0, wx.EXPAND, 0)
        self.voigtFitPane.SetSizer(sizer_9_copy_2)
        sizer_9_copy.Add(self.label_7_copy, 0, 0, 0)
        grid_sizer_4_copy.Add(self.label_5_copy, 0, 0, 0)
        grid_sizer_4_copy.Add(self.label_11_copy, 0, 0, 0)
        grid_sizer_4_copy.Add((20, 20), 0, 0, 0)
        grid_sizer_4_copy.Add(self.label_12_copy, 0, 0, 0)
        grid_sizer_4_copy.Add((20, 20), 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussATextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussSigATextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussMuACheckBox, 0, wx.ALIGN_RIGHT, 0)
        grid_sizer_4_copy.Add(self.dgaussMuATextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.singleFitAButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4_copy.Add(self.dgaussBTextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussSigBTextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussMuBCheckBox, 0, wx.ALIGN_RIGHT, 0)
        grid_sizer_4_copy.Add(self.dgaussMuBTextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add(self.singleFitBButton, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4_copy.Add((20, 20), 0, 0, 0)
        grid_sizer_4_copy.Add(self.label_13, 0, 0, 0)
        grid_sizer_4_copy.Add(self.dgaussCCheckBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_4_copy.Add(self.dgaussCTextCtrl, 0, 0, 0)
        grid_sizer_4_copy.Add((20, 20), 0, 0, 0)
        sizer_9_copy.Add(grid_sizer_4_copy, 1, wx.EXPAND, 0)
        sizer_10_copy.Add(self.label_1_copy_1, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy.Add(self.dgaussTolTextCtrl, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_9_copy.Add(sizer_10_copy, 0, wx.EXPAND, 0)
        self.doubleGaussFitPane.SetSizer(sizer_9_copy)
        sizer_11.Add(self.label_17, 0, 0, 0)
        grid_sizer_6.Add(self.label_18, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerATextCtrl, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerFixedACheckBox, 0, 0, 0)
        grid_sizer_6.Add(self.label_19, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerXcTextCtrl, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerFixedXcCheckBox, 0, 0, 0)
        grid_sizer_6.Add(self.label_20, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerAlphaTextCtrl, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerFixedAlphaCheckBox, 0, 0, 0)
        grid_sizer_6.Add(self.label_21, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerBetaTextCtrl, 0, 0, 0)
        grid_sizer_6.Add(self.twoPowerFixedBetaCheckBox, 0, 0, 0)
        sizer_11.Add(grid_sizer_6, 0, 0, 0)
        sizer_12.Add(self.label_22, 0, 0, 0)
        sizer_12.Add(self.twoPowerTolTextCtrl, 0, 0, 0)
        sizer_11.Add(sizer_12, 0, wx.EXPAND, 0)
        self.twoPowerFitPane.SetSizer(sizer_11)
        self.fitNotebook.AddPage(self.splineFitPane, "Spline")
        self.fitNotebook.AddPage(self.polyFitPane, "Polynomial")
        self.fitNotebook.AddPage(self.powerLawFitPane, "Power Law")
        self.fitNotebook.AddPage(self.blackbodyFitPane, "Blackbody")
        self.fitNotebook.AddPage(self.gaussFitPane, "Gaussian")
        self.fitNotebook.AddPage(self.lorentzFitPane, "Lorentzian")
        self.fitNotebook.AddPage(self.voigtFitPane, "Voigt")
        self.fitNotebook.AddPage(self.doubleGaussFitPane, "Double Gaussian")
        self.fitNotebook.AddPage(self.twoPowerFitPane, "2-Power Model")
        sizer_7.Add(self.fitNotebook, 0, wx.EXPAND, 0)
        sizer_7.Add(self.fitOutputText, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_7)
        sizer_7.Fit(self)
        # end wxGlade

    def _updatePreview(self,drawtocanvas=True,relim=True):
        if self.__drawcode > 0:
            try:
                res=self.fitData(extras=True)
                sl=['RMS Deviation:']
                sl.append(str(res[1]))
                sl.append('\nParameters:')
                for k,v in res[2].iteritems():
                    sl.append(k+': '+str(v))
                sl.append('\nExtra Information:')
                sl.append(str(res[3:]))
                self.fitOutputText.Value='\n'.join(sl)
            except Exception,e:
                self.fitOutputText.Value='Problem with function fit!\n'+str(e)
                self.__drawcode=0

        if not relim:
            oldxlim=self.axes.get_xlim()
            oldylim=self.axes.get_ylim()

        self.axes.clear()
        if self.log:
            if self.log is True:
                self.log = 'xy'
            if self.log == 'xy':
                plotfunc=self.axes.loglog
            elif self.log == 'x':
                plotfunc=self.axes.semilogx
            elif self.log == 'y':
                plotfunc=self.axes.semilogy
            else:
                raise ValueError('Illegal value for self.log')
        else:
            plotfunc=self.axes.plot
        #draw codes: 0=input data,1=fit,2=residuals,3=all
        if self.__drawcode==0:
            self.axes.clear()
            self.axes.set_title('Input Data')
            plotfunc(self.__x,self.__y,self.datastyle)
            if self.__selections is not None:
                for l,u in self.__selections:
                    self.axes.axvspan(self.__x[l],self.__x[u],ec=self.seledgecolor,fc=self.selfacecolor,alpha=self.selalpha)

        elif self.__drawcode==1:
            self.axes.set_title('Data and Fit')
            #overplot fit to actual data
            plotfunc(self.__x,self.__y,self.datastyle,label='Data')
            plotfunc(self.__x,res[0],self.fitstyle,label='Fit')
            self.axes.legend()
            if self.__selections is not None:
                for l,u in self.__selections:
                    self.axes.axvspan(self.__x[l],self.__x[u],ec=self.seledgecolor,fc=self.selfacecolor,alpha=self.selalpha)

        elif self.__drawcode==2:
            self.axes.set_title('Residuals')
            #plot residuals
            plotfunc(self.__x,self.__y-res[0],self.resstyle)
            if self.__selections is not None:
                for l,u in self.__selections:
                    self.axes.axvspan(self.__x[l],self.__x[u],ec=self.seledgecolor,fc=self.selfacecolor,alpha=self.selalpha)

        elif self.__drawcode==3:
            self.axes.set_title('Data, Fit, and Residuals')
            #overplot fit to actual data and residuals
            plotfunc(self.__x,self.__y,self.datastyle,label='Data')
            plotfunc(self.__x,res[0],self.fitstyle,label='Fit')
            #resoffset=self.__y.mean()-(self.axes.get_ylim()[1]-self.axes.get_ylim()[0])/4
            resoffset=self.__y.mean()
            plotfunc(self.__x,self.__y-res[0]+resoffset,self.resstyle,label='Res+%1.1e' %resoffset)
            self.axes.legend()
            if self.__selections is not None:
                for l,u in self.__selections:
                    self.axes.axvspan(self.__x[l],self.__x[u],ec=self.seledgecolor,fc=self.selfacecolor,alpha=self.selalpha)
        else:
            raise Exception('internal FitPanel error - unrecognized drawcode')

        if not relim:
            self.axes.set_xlim(oldxlim)
            self.axes.set_ylim(oldylim)

        if self.__sstool is not None:
            #TODO:check for memory leaks - if this is eventually fixed, re-enable self.__sstool.visible = not self.previewNavBar.getToolState() below
            self.__sstool.visible=False
            if not self.previewNavBar.getToolState():
                self.__sstool=SpanSelector(self.axes,self.onSelect,'horizontal',useblit=True)

        if drawtocanvas:
            self.previewCanvas.draw()

    def __smartInitialParams(self,x,y,vertoff=False):
        from math import pi

        x=numpy.array(x)
        y=numpy.array(y)

        centroid=(x*y).sum()/y.sum()
        centermoment2=abs(numpy.sum(y*(x-centroid)**2)/y.sum())
        maxval=max(y)

        gaucorr=0 if centermoment2==0 else (2*pi*centermoment2)**-0.5
        lorcorr=0 if centermoment2==0 else 1./pi/centermoment2

        self.gaussATextCtrl.Value = str(gaucorr*maxval)
        self.gaussSigTextCtrl.Value = str(centermoment2)
        self.gaussMuTextCtrl.Value = str(centroid)

        self.lorATextCtrl.Value = str(lorcorr*maxval)
        self.lorScaleTextCtrl.Value = str(centermoment2)
        self.lorMuTextCtrl.Value = str(centroid)

        self.voigtATextCtrl.Value = str(lorcorr*gaucorr*maxval)
        self.voigtSigTextCtrl.Value = str(centermoment2)
        self.voigtScaleTextCtrl.Value = str(centermoment2)
        self.voigtMuTextCtrl.Value = str(centroid)

        self.dgaussATextCtrl.Value = str(gaucorr*maxval)
        self.dgaussSigATextCtrl.Value = str(centermoment2)
        self.dgaussMuATextCtrl.Value = str(centroid)
        self.dgaussBTextCtrl.Value = str(-gaucorr*maxval)
        self.dgaussSigBTextCtrl.Value = str(2*centermoment2)
        self.dgaussMuBTextCtrl.Value = str(centroid)

        if vertoff:
            meanval=y.mean()
            self.gaussCTextCtrl.Value = str(meanval)
            self.lorCTextCtrl.Value = str(meanval)
            self.voigyCTextCtrl.Value = str(meanval)
            self.dgaussCTextCtrl.Value = str(meanval)
            
    def setData(self,x,y,newPars=True):
        """
        copies in the data x and y and stores it for fitting x can be None,
        which will result in a linear [0,len(y)) axis.

        also redraws the new data
        """
        if x is None:
            self.__x=numpy.arange(len(y))
        else:
            self.__x=numpy.array(x)
        self.__y=numpy.array(y)
        
        #Change the selection back to all for the new data
        if self.__sstool is not None:
            self.__sstool.visible=False
        self.__sstool=None
        self.__selections=None
        self.selectionRadioBox.SetSelection(0)
        
        self._updatePreview()

        self.__smartInitialParams(x,y)

    def getData(self):
        """
        copies the data and returns it
        returns data as (x,y)
        """
        return (self.__x.copy(),self.__y.copy())

    def getSelectedData(self):
        """
        generates copies of the data from the current selection

        returns as (xvalues,yvalues)
        """
        if self.__selections is None:
            return (self.__x.copy(),self.__y.copy())

        x=numpy.concatenate([self.__x[l:u] for l,u in self.__selections])
        y=concatenate([self.__y[l:u] for l,u in self.__selections])
        return x,y

    def fitData(self,x=None,y=None,selection=None,extras=False):
        """
        if x, y,and selection are None, the selection range from self will be used
        if just x is None, a linear array will be created for fitting and the self
        values will be ignored
        """
        import fitutils
        if x is None and y is None:
            x=self.__x
            y=self.__y

        sels=self.__selections if selection is None else selection

        if sels is not None and len(sels)==0:
            constmean=n.ndarray(len(y))
            constmean.fill(y.mean())

            return (constmean,n.std(constmean-y))
        else:
            pane=self.fitNotebook.GetCurrentPage()

            if pane is self.splineFitPane:
                deg=int(self.splineDegTextCtrl.GetValue())
                #seg=int(self.splineSegTextCtrl.GetValue())
                smoothing=float(self.splineSmoothingTextCtrl.GetValue())
                #return fitSplines(x,y,segments=seg,degree=deg,smfactor=smoothing,fitranges=sels)
                return fitutils.fitSplines(x,y,degree=deg,smfactor=smoothing,fitranges=sels)

##            elif pane is self.chebyshevFitPane: ##TODO:FIX
##                tol=float(self.chebTolTextCtrl.GetValue())
##                deg=int(self.chebDegTextCtrl.GetValue())
##                type=self.chebTypeRadioBox.GetStringSelection().lower()
##                return fitutils.fitChebyshevPolynomial(x,y,degree=deg,chebyshevtype=type,fitranges=sels,tol=tol)

            elif pane is self.powerLawFitPane:
                tol=float(self.powerTolTextCtrl.GetValue())
                A=float(self.powerOffTextCtrl.GetValue())
                alpha=float(self.powerSlopeTextCtrl.GetValue())
                return fitutils.fitPowerLaw(x,y,A,alpha,fitranges=sels,tol=tol)
            
            elif pane is self.polyFitPane:
                tol=float(self.polyTolTextCtrl.GetValue())
                deg=int(self.polyDegTextCtrl.GetValue())
                type=self.polyTypeRadioBox.GetStringSelection().lower()
                if type== 'simple':
                    return fitutils.fitPolynomial(x,y,deg,icoeffs=None,fitranges=sels,tol=tol)
                elif type == 'legendre':
                    return fitutils.fitLegendrePolynomial(x,y,deg,icoeffs=None,fitranges=sels,tol=tol)
                else:
                    type=type.replace('chebyshev ','')
                    return fitutils.fitChebyshevPolynomial(x,y,deg,type,icoeffs=None,fitranges=sels,tol=tol)
            
            elif pane is self.blackbodyFitPane:
                xtol=float(self.bbXtolTextCtrl.GetValue())
                ftol=float(self.bbFtolTextCtrl.GetValue())
                wl=self.bbTypeRadioBox.GetStringSelection() == 'Wavelength'
                T0=float(self.bbInitTempTextCtrl.GetValue())
                A0=self.bbInitA0.GetValue()
                A0=None if A0.strip()=='' else float(A0)
                return fitutils.fitPlanckBB(x,y,T0,A0,wl,fitranges=sels,tol=(xtol,ftol))

            elif pane is self.gaussFitPane:
                tol=float(self.gaussTolTextCtrl.GetValue())
                A=float(self.gaussATextCtrl.GetValue())
                sigma=float(self.gaussSigTextCtrl.GetValue())
                mu=float(self.gaussMuTextCtrl.GetValue())
                C=float(self.gaussCTextCtrl.GetValue())

                fixed=[]
                if not self.gaussCCheckBox.GetValue():
                    fixed.append('C')
                if not self.gaussMuCheckBox.GetValue():
                    fixed.append('mu')

                return fitutils.fitGaussian(x,y,A0=A,sigma0=sigma,mu0=mu,C0=C,fitranges=sels,tol=tol,fixedpars=fixed)

            elif pane is self.lorentzFitPane:
                tol=float(self.lorTolTextCtrl.GetValue())
                A=float(self.lorATextCtrl.GetValue())
                xs=float(self.lorScaleTextCtrl.GetValue())
                mu=float(self.lorMuTextCtrl.GetValue())
                C=float(self.lorCTextCtrl.GetValue())

                fixed=[]
                if not self.lorCCheckBox.GetValue():
                    fixed.append('C')
                if not self.lorMuCheckBox.GetValue():
                    fixed.append('mu')

                return fitutils.fitLorentzian(x,y,A0=A,xs0=xs,mu0=mu,C0=C,fitranges=sels,tol=tol,fixedpars=fixed)

            elif pane is self.voigtFitPane:
                tol=float(self.voigtTolTextCtrl.GetValue())
                A=float(self.voigtATextCtrl.GetValue())
                G=float(self.voigtGTextCtrl.GetValue())
                L=float(self.voigtLTextCtrl.GetValue())
                sigma=float(self.voigtSigTextCtrl.GetValue())
                xs=float(self.voigtScaleTextCtrl.GetValue())
                mu=float(self.voigtMuTextCtrl.GetValue())
                C=float(self.gaussCTextCtrl.GetValue())

                fixed=[]
                if not self.voigtCCheckBox.GetValue():
                    fixed.append('C')
                if not self.voigtMuCheckBox.GetValue():
                    fixed.append('mu')
                if self.voigtFixACheckBox.GetValue():
                    fixed.append('A')
                if self.voigtFixGCheckBox.GetValue():
                    fixed.append('G')
                if self.voigtFixLCheckBox.GetValue():
                    fixed.append('L')

                return fitutils.fitVoigt(x,y,A0=A,G0=G,L0=L,sigma0=sigma,xs0=xs,mu0=mu,C0=C,fitranges=sels,tol=tol,fixedpars=fixed)

            elif pane is self.doubleGaussFitPane:
                tol=float(self.gaussTolTextCtrl.GetValue())
                A=float(self.dgaussATextCtrl.GetValue())
                B=float(self.dgaussBTextCtrl.GetValue())
                sigmaA=float(self.dgaussSigATextCtrl.GetValue())
                sigmaB=float(self.dgaussSigBTextCtrl.GetValue())
                muA=float(self.dgaussMuATextCtrl.GetValue())
                muB=float(self.dgaussMuBTextCtrl.GetValue())
                C=float(self.dgaussCTextCtrl.GetValue())

                fixed=[]
                if not self.dgaussCCheckBox.GetValue():
                    fixed.append('C')
                if not self.dgaussMuACheckBox.GetValue():
                    fixed.append('muA')
                if not self.dgaussMuBCheckBox.GetValue():
                    fixed.append('muB')

                return fitutils.fitDoubleGaussian(x,y,A0=A,B0=B,sigmaA0=sigmaA,
                        sigmaB0=sigmaB,C0=C,muA0=muA,muB0=muB,fitranges=sels,tol=tol,fixedpars=fixed)
            elif pane is self.twoPowerFitPane:
                tol=float(self.twoPowerTolTextCtrl.GetValue())
                A=float(self.twoPowerATextCtrl.GetValue())
                xc=float(self.twoPowerXcTextCtrl.GetValue())
                alpha=float(self.twoPowerAlphaTextCtrl.GetValue())
                beta=float(self.twoPowerBetaTextCtrl.GetValue())
                
                fixed=[]
                if self.twoPowerFixedACheckBox.GetValue():
                    fixed.append('A')
                if self.twoPowerFixedXcCheckBox.GetValue():
                    fixed.append('xc')
                if self.twoPowerFixedAlphaCheckBox.GetValue():
                    fixed.append('alpha')
                if self.twoPowerFixedBetaCheckBox.GetValue():
                    fixed.append('beta')
                return fitutils.fit2PowerModel(x,y,A=A,xc=xc,alpha=alpha,beta=beta,fixedpars=fixed,fitranges=sels,tol=tol)
            else:
                raise Exception('Should never hit this point! something very wrong in the parameters notebook')

    def onToolToggle(self,state): #custom navbar event
        if self.__sstool is not None:
            self.__sstool.visible = not state
            
    def onChar(self,event):
        key = chr(event.GetKeyCode())
        if key=='z':
            self.previewNavBar.clickZoom()
        elif key=='p':
            self.previewNavBar.clickPan()
        elif key=='a':
            self.selectionRadioBox.SetSelection(0)
            self.onSelectChange(event)
        elif key=='i':
            self.selectionRadioBox.SetSelection(1)
            self.onSelectChange(event)
        elif key=='e':
            self.selectionRadioBox.SetSelection(2)
            self.onSelectChange(event)

    def onPrevReset(self, event): # wxGlade: FitPanel.<event_handler>
        #relim=(self.__drawcode == 2 or self.__drawcode ==3)
        self.__drawcode=0
        self._updatePreview(relim=True)

    def onPrevFit(self, event): # wxGlade: FitPanel.<event_handler>
        relim=self.__drawcode == 2
        self.__drawcode=1
        self._updatePreview(relim=relim)

    def onPrevRes(self, event): # wxGlade: FitPanel.<event_handler>
        #relim=not self.__drawcode == 2
        self.__drawcode=2
        self._updatePreview(relim=True)

    def onDFR(self, event): # wxGlade: FitPanel.<event_handler>
        relim = self.__drawcode == 2
        self.__drawcode=3
        self._updatePreview(relim=relim)

    def onSelectChange(self, event): # wxGlade: FitPanel.<event_handler>
        if self.__sstool is not None:
            self.__sstool.visible=False
            
        dorelim = False

        sel=self.selectionRadioBox.GetStringSelection()
        if sel == 'All':
            dorelim = True
            self.__sstool=None
            self.__selections=None
        elif sel == 'Include':
            if self.__sstool is None:
                self.__sstool=SpanSelector(self.axes,self.onSelect,'horizontal',useblit=True)
            if self.__selections is None:
                self.__selections=[]
        elif sel == 'Exclude':
            if self.__sstool is None:
                self.__sstool=SpanSelector(self.axes,self.onSelect,'horizontal',useblit=True)
            if self.__selections is None:
                self.__selections=[(0,len(self.__y)-1)]
        else:
            raise Exception('Should never hit here - selectionRadioBox has a mis-named option')

        self._updatePreview(relim=dorelim) #note that the SpanSelector may be disabled here

    def onSelect(self,xmin,xmax): #handler for SpanSelector
        #find cloest indecies
        offsets=abs(self.__x-xmin)
        lind=numpy.where(offsets==min(offsets))[0][0]
        offsets=abs(self.__x-xmax)
        uind=numpy.where(offsets==min(offsets))[0][0]

        sel=self.selectionRadioBox.GetStringSelection()

        #find which selection indecies the input bounds are in - positive indsel means
        #the bound is in that selection, negative means its outside but below -sel-1
        lindsel=-1
        uindsel=-1
        for i,(ls,us) in enumerate(self.__selections):
            if lind >= us:
                lindsel = -i-2
            elif ls < lind and lind < us:
                lindsel = i
            #final implied case:
            #elif lind <=ls:
            #    pass
            if uind >= us:
                uindsel = -i-2
            elif ls < uind and uind < us:
                uindsel = i
            #final implied case:
            #elif uind <=ls:
            #    pass
            
            
        #check sort
        #last = 0
        #for ls,us in self.__selections:
        #    assert ls-last>0,'sort problem '+str(self.__selections)
        #    last = ls
            
        if sel == 'Include':
            if lindsel == uindsel:
                #note that this is a shortcut - the else case will work for these values, too
                if lindsel < 0:
                    #in a blank space - just insert into the right place to keep the list sorted
                    self.__selections.insert(-lindsel-1,(lind,uind))
                #else in an already selected area - ignore
            else:
                if lindsel>=0 and uindsel>=0:
                    self.__selections.insert(uindsel+1,(self.__selections[lindsel][0],self.__selections[uindsel][1]))
                elif lindsel >=0:
                    uindsel = -uindsel-2
                    self.__selections.insert(uindsel+1,(self.__selections[lindsel][0],uind))
                elif uindsel >=0:
                    lindsel = -lindsel-1
                    self.__selections.insert(uindsel+1,(lind,self.__selections[uindsel][1]))
                else:
                    uindsel = -uindsel-2
                    lindsel = -lindsel-1
                    self.__selections.insert(uindsel+1,(lind,uind))

                #remove the ranges between the bottom and the newly added range
                del self.__selections[lindsel:uindsel+1]


            #this is more efficient, but doesn't work if switching from exclude to include
            #self.axes.axvspan(xmin,xmax,ec='k',fc='r',alpha=0.5)
            #self.previewCanvas.draw()
        elif sel == 'Exclude':
            if lindsel == uindsel:
                if lindsel>=0:
                    #cut up this selection into two pieces
                    lold,uold=self.__selections.pop(lindsel)
                    self.__selections.insert(lindsel,(uind,uold))
                    self.__selections.insert(lindsel,(lold,lind))

            else:
                #shorten any selection that the bounds might fall inside
                if lindsel >= 0:
                    self.__selections[lindsel]=(self.__selections[lindsel][0],lind)
                if uindsel >= 0:
                    self.__selections[uindsel]=(uind,self.__selections[uindsel][1])
                #remove any selections that are wholly interior to the selection
                lselbound=lindsel+1 if lindsel >= 0 else -lindsel-1
                uselbound=uindsel if uindsel >= 0 else -uindsel-1
                del self.__selections[lselbound:uselbound]

        else:
            raise Excepition('Should never reach here - selectionRadioBox has a mis-named option')

        self._updatePreview(relim=False)


    def onParamEnter(self, event): # wxGlade: FitPanel.<event_handler>
        self._updatePreview(relim=False)

    def onSingleFitA(self, event): # wxGlade: FitPanel.<event_handler>
        from fitutils import fitGaussian
        x=self.__x
        y=self.__y
        sels=self.__selections
        
        tol=float(self.gaussTolTextCtrl.GetValue())
        A=float(self.dgaussATextCtrl.GetValue())
        sigmaA=float(self.dgaussSigATextCtrl.GetValue())
        muA=float(self.dgaussMuATextCtrl.GetValue())
        C=float(self.dgaussCTextCtrl.GetValue())

        fixed=[]
        if not self.dgaussCCheckBox.GetValue():
            fixed.append('C')
        if not self.dgaussMuACheckBox.GetValue():
            fixed.append('mu')
            
        pars = fitGaussian(x,y,A0=A,sigma0=sigmaA,mu0=muA,C0=C,fitranges=sels,tol=tol,fixedpars=fixed)[2]
        
        self.dgaussATextCtrl.SetValue(str(pars['A']))
        self.dgaussSigATextCtrl.SetValue(str(pars['sigma']))
        self.dgaussMuATextCtrl.SetValue(str(pars['mu']))
        self.dgaussCTextCtrl.SetValue(str(pars['C']))

    def onSingleFitB(self, event): # wxGlade: FitPanel.<event_handler>
        from fitutils import fitGaussian
        x=self.__x
        y=self.__y
        sels=self.__selections
        
        tol=float(self.gaussTolTextCtrl.GetValue())
        B=float(self.dgaussBTextCtrl.GetValue())
        sigmaB=float(self.dgaussSigBTextCtrl.GetValue())
        muB=float(self.dgaussMuBTextCtrl.GetValue())
        C=float(self.dgaussCTextCtrl.GetValue())
        
        fixed=[]
        if not self.dgaussCCheckBox.GetValue():
            fixed.append('C')
        if not self.dgaussMuBCheckBox.GetValue():
            fixed.append('mu')
            
        pars = fitGaussian(x,y,A0=B,sigma0=sigmaB,mu0=muB,C0=C,fitranges=sels,tol=tol,fixedpars=fixed)[2]
        
        self.dgaussBTextCtrl.SetValue(str(pars['A']))
        self.dgaussSigBTextCtrl.SetValue(str(pars['sigma']))
        self.dgaussMuBTextCtrl.SetValue(str(pars['mu']))
        self.dgaussCTextCtrl.SetValue(str(pars['C']))
        

# end of class FitPanel


class FitDialog(wx.Dialog):
    def __init__(self,x,y,usepages,*args, **kwds):
        """
        x and y are the initial data to be used (if x is None, will generate
        a linear 0-indexed x-axis)
        
        see FitPanel for possible usepages
        """
        self.__fitResults=None

        # begin wxGlade: FitDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER|wx.THICK_FRAME
        wx.Dialog.__init__(self, *args, **kwds)
        self.fitPane = FitPanel(x, y, usepages, self, -1)
        self.okButton = wx.Button(self, wx.ID_OK, "Accept Fit")
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.onAccept, id=wx.ID_OK)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: FitDialog.__set_properties
        self.SetTitle("Function Fitting")
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: FitDialog.__do_layout
        sizer_3 = wx.BoxSizer(wx.VERTICAL)
        sizer_4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3.Add(self.fitPane, 1, wx.EXPAND, 0)
        sizer_4.Add(self.okButton, 0, wx.RIGHT|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_4.Add(self.cancelButton, 0, wx.LEFT|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_3.Add(sizer_4, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 8)
        self.SetSizer(sizer_3)
        sizer_3.Fit(self)
        self.Layout()
        # end wxGlade

    def onAccept(self, event): # wxGlade: FitDialog.<event_handler>
        self.__fitResults=self.fitPane.fitData(extras=True)
        event.Skip()

    def getAllResults(self):
        return self.__fitResults

    def getFit(self):
        """
        returns the fit results generated by this dialog.  Will be None if no fit is
        ready yet.
        """
        return None if self.__fitResults is None else self.__fitResults[0]

    def getFitRMSDev(self):
        """
        gets the sd of the fit
        """
        return None if self.__fitResults is None else self.__fitResults[1]

    def getFitParams(self):
        """
        gets a dictionary with the parameters used in the fit
        """
        return None if self.__fitResults is None else self.__fitResults[2]

    def getFitExtras(self):
        """
        returns the extra information generated from the fit
        """
        return None if self.__fitResults is None else self.__fitResults[3:]

    def setData(self,x,y):
        return self.fitPane.setData(x,y)
    
    def setStyles(self,data,fit=None,res=None):
        if data is not None:
            self.fitPane.datastyle = data
        if fit is not None:
            self.fitPane.fitstyle = fit
        if res is not None:
            self.fitPane.resstyle = res
        self.fitPane._updatePreview(True,False)
        
    def setLog(self,log):
        self.fitPane.log = log
        self.fitPane._updatePreview(True,False)

# end of class FitDialog
def fitsimple(*args,**kwargs):
    """
    alias for simplestfit
    """
    simplestfit(*args,**kwargs)
    
def simplestfit(*args,**kwargs):
    """
    this is a convinience function to simplify data-fitting - either call as
    simplestfit(y) or simplestfit(x,y) , and the rest will be taken care of.

    specifics of the return values depend on the type of fit, but it will always
    be a tuple of the form (fitted_curve,rms_deviation,parameter_dict,extra_info)
    """
    from wx._core import PyNoAppError
    from numpy import arange

    if len(args)<1 or len(args)>2:
        raise TypeError('Got 0 or more than 2 arguments')

    if len(args)==1:
        y=args[0]
        x=arange(0,len(y))
    else:
        x=args[0]
        y=args[1]

    res=[]
    try:
        dlg=FitDialog(x,y,None,None,-1)
        df=ff=rf=None
        if 'fmt' in kwargs:
            df = kwargs['fmt']
        if 'fitfmt' in kwargs:
            ff = kwargs['fitfmt']
        if 'resfmt' in kwargs:
            rf = kwargs['resfmt']
        dlg.setStyles(df,ff,rf)
        if 'log' in kwargs:
            dlg.setLog(kwargs['log'])
        dlg.ShowModal()
        res.append(dlg.getAllResults())
    except PyNoAppError:
        from wx import PySimpleApp
        app=PySimpleApp()
        class FitFrame(wx.Frame):
            def __init__(self,x,y,*args,**kwargs):
                wx.Frame.__init__(self,*args,**kwargs)

                self.p=FitPanel(x,y,None,self,-1)
                self.Bind(wx.EVT_CLOSE,self.onClose)

            def onClose(self,event):
                res.append(self.p.fitData(extras=True))
                event.Skip()

        fr=FitFrame(x,y,None)

        fr.Show(True)
        app.MainLoop()

    return res[0]


if __name__ == "__main__":
    from numpy.random import randn
    from numpy import exp,linspace,array
    from math import pi,sqrt
    import sys
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option('-b','--blackbody', dest="bb",help='make spectrum with temperature temp',default=None)
    parser.add_option('-g','--gaussian',dest='gauss',help='make a gaussian with the specified SD',default=None)
    parser.add_option('-d','--doublegaussian',dest='dgs',help='make a double gaussian with the parameters "SD1,SD2,amp2"',default=None)
    parser.add_option('-r','--random',dest='rnd',help='apply a normally random perturbation with the requested amplitude',default='0')
    parser.add_option('-a','--amplitude',dest='amp',help='multiply the result be the specified factor',default='1')
    parser.add_option('-x','--xaxis',dest='xax',help='specify the xaxis as xlower,xupper,numsamples',default='-1,1,100')
    options, args = parser.parse_args()

    xaxsl=options.xax.split(',')
    x=linspace(float(xaxsl[0]),float(xaxsl[1]),int(xaxsl[2]))

    h=6.626068E-11 #angstroms^2 g/s=6.626068E-27 cm^2 g/s = erg*s
    c=2.99792458E18 #angstroms/sec=3E10 cm/sec
    k=1.3806503 #angstrom^2 g/s^2/K  1.3806503E-16 cm^2 g/s^2/K=erg/K

    typen=array([options.bb is not None,options.gauss is not None,options.dgs is not None]).sum()
    if  typen>1:
        print 'Can only specify one type of spectrum to test - asked for 2 or more, exiting...'
        sys.exit(1)

    if typen < 1:
        print 'Need to specify example spectrum type as either -b, -g, or -d, exiting...'
        sys.exit(1)
    if options.bb is not None:
        y=x**-5/(exp((h*c/(k*float(options.bb)*x)))-1)
    elif options.gauss is not None:
        y=exp(-x**2/(2*float(options.gauss)))*float(options.gauss)**-0.5/sqrt(2*pi)
    elif options.dgs is not None:
        dgsl=options.dgs.split(',')
        s1=float(dgsl[0])
        s2=float(dgsl[1])
        amp2=float(dgsl[2])
        y=exp(-x**2/(2*s1))*s1**-0.5/sqrt(2*pi)+amp2*exp(-x**2/(2*s2))*s2**-0.5/sqrt(2*pi)

    y*=float(options.amp)
    y+=float(options.rnd)*randn(len(x))

    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()
    fitDialog = FitDialog(x,y,None,None, -1, "")
    app.SetTopWindow(fitDialog)
    fitDialog.Show()
    app.MainLoop()
